<?php
declare(strict_types=1);

namespace ImageResizer\Lib;

use ImageResizer\Models\Image;
use ImageResizer\Models\ResizeParams;
use Psr\Log\AbstractLogger;

class SqliteImageDb implements ImageDb {
    public function __construct(
        private readonly \PDO $db,
        private readonly AbstractLogger $logger,
    ) {
        $this->up();
    }

    private function up() {
        $this->db->exec('CREATE TABLE IF NOT EXISTS images (
            id INTEGER PRIMARY KEY,
            path TEXT NOT NULL,
            width INTEGER NULLABLE,
            height INTEGER NULLABLE,
            quality INTEGER NULLABLE,
            content TEXT NOT NULL
        )');

        // Create a unique index on the path and params
        $this->db->exec('CREATE UNIQUE INDEX IF NOT EXISTS images_path_width_height_quality ON images (path, width, height, quality)');
    }

    public function findImage(string $path, ResizeParams $params): Image | null {
        $query = 'SELECT * FROM images WHERE path = :path';
        $query .= $params->width ? ' AND width = :width' : ' AND width IS NULL';
        $query .= $params->height ? ' AND height = :height' : ' AND height IS NULL';
        $query .= $params->quality ? ' AND quality = :quality' : ' AND quality IS NULL';

        $stmt = $this->db->prepare($query);
        $stmt->bindParam(':path', $path);
        if ($params->width !== null) $stmt->bindParam(':width', $params->width);
        if ($params->height !== null) $stmt->bindParam(':height', $params->height);
        if ($params->quality !== null) $stmt->bindParam(':quality', $params->quality);

        $this->logger->info("Querying images", ['query' => $stmt->queryString]);

        $stmt->execute();

        $result = $stmt->fetchObject();

        return $result ? Image::fromString(base64_decode($result->content)) : null;
    }

    public function saveImage(
        string $path,
        ResizeParams $params,
        string $content,
    ): void {
        $stmt = $this->db->prepare("INSERT INTO images (path, width, height, quality, content) VALUES (:path, :width, :height, :quality, :content)"
);
        $stmt->execute([
            ':path' => $path,
            ':width' => $params->width,
            ':height' => $params->height,
            ':quality' => $params->quality,
            ':content' => $content
        ]);
    }
}