<?php
declare(strict_types=1);


namespace ImageResizer\Models;

class Image
{
    public function __construct(
        private \GdImage $image,
        private int      $type,
    )
    {
    }

    public static function fromUrl(string $url): self
    {
        $remote_source = file_get_contents($url);

        if (!$remote_source) {
            throw new \Exception("Remote image does not exist");
        }

        return self::fromString($remote_source);
    }

    public static function fromString(string $str): self
    {
        $image = imagecreatefromstring($str);

        if (!$image) {
            throw new \Exception("Failed to parse image");
        }

        [2 => $type] = getimagesizefromstring($str);
        return new self($image, $type);
    }

    public function getWidth(): int
    {
        return imagesx($this->image);
    }

    public function getHeight(): int
    {
        return imagesy($this->image);
    }

    public function getType(): int
    {
        return $this->type;
    }

    public function getMime(): string
    {
        return image_type_to_mime_type($this->type);
    }

    public function getExt(): string
    {
        return image_type_to_extension($this->type);
    }

    public function raw(): \GdImage
    {
        return $this->image;
    }

    public function setType(int $type): void
    {
        $this->type = $type;
    }

    public function scale(int $ratio): void
    {
        $this->image = imagescale($this->image, $this->getWidth() * $ratio, $this->getHeight() * $ratio);
    }

    public function toWidth(int $width): void
    {
        if ($width < $this->getWidth()) {

            $ratio = $width / $this->getWidth();
            $height = intval(floor($this->getHeight() * $ratio));

            $this->image = imagescale($this->image, $width, $height);
        }
    }

    public function encode(int $quality): string
    {
        ob_start();
        $type = $this->clientSupportsWebp() ? IMAGETYPE_WEBP : $this->type;

        switch ($type) {
            case IMAGETYPE_PNG:
                imagepng($this->image, null, $quality);
                break;
            case IMAGETYPE_GIF:
                imagegif($this->image);
                break;
            case IMAGETYPE_JPEG:
                imagejpeg($this->image, null, $quality);
                break;
            case IMAGETYPE_WEBP:
                imagewebp($this->image, null, $quality);
                break;
        }

        $image_data = ob_get_contents();
        ob_end_clean();

        return base64_encode($image_data);
    }

    private function clientSupportsWebp(): bool {
        return str_contains($_SERVER['HTTP_ACCEPT'], 'image/webp');
    }
}